@@ -21,6 +21,7 @@ import os
 import socket
 import threading
 import traceback
+import uuid
 import weakref
 
 from http import HTTPStatus
@@ -38,6 +39,7 @@ from akit.friendlyidentifier import FriendlyIdentifier
 from akit.networking.constants import AKitHttpHeaders, HTTP1_1_LINESEP, HTTP1_1_END_OF_HEADER
 from akit.networking.interfaces import get_interface_for_ip
 from akit.networking.resolution import get_arp_table, refresh_arp_table
+from akit.networking.trafficcapturecontext import TrafficCaptureContext
 
 from akit.interop import upnp as upnp_module
 from akit.interop.landscaping.landscapedevice import LandscapeDevice
@@ -89,6 +91,7 @@ def http_parse_header(header_content):
             headers[hname] = hval
     return req_line, headers
 
+
 class UpnpCoordinator(CoordinatorBase):
     """
         The UpnpCoordinator utilizes the expected device declarations of type 'network/upnp' to establish and maintain connectivity
@@ -165,6 +168,9 @@ class UpnpCoordinator(CoordinatorBase):
         # A reverse lookup table that can lookup the device hint from its USN
         self._cl_usn_to_hint = {}
 
+        # Callback capture management dictionaries
+        self._cl_callback_traffic_capture_from = {}
+
         # =========================== Queue Lock Variables ==========================
         # Variables that manage the work queue and dispatching of work to worker threads
         self._queue_lock = threading.RLock()
@@ -197,6 +203,38 @@ class UpnpCoordinator(CoordinatorBase):
 
         return wlist
 
+    def create_callback_traffic_capture_context(self, fromip: str):
+        
+        identifier = uuid.uuid4()
+        context = TrafficCaptureContext(self, identifier, fromip, self.destroy_callback_traffic_capture_context)
+
+        self._coord_lock.acquire()
+        try:
+            if fromip in self._cl_callback_traffic_capture_from:
+                self._cl_callback_traffic_capture_from[fromip][context.identifier] = context
+            else:
+                self._cl_callback_traffic_capture_from[fromip] = {context.identifier: context}
+        finally:
+            self._coord_lock.release()
+
+        return context
+
+    def destroy_callback_traffic_capture_context(self, context: TrafficCaptureContext):
+
+        fromip = context.fromip
+        identifier = context.identifier
+
+        self._coord_lock.acquire()
+        try:
+            if fromip in self._cl_callback_traffic_capture_from:
+                fromip_table = self._cl_callback_traffic_capture_from[fromip]
+                if identifier in fromip_table:
+                    del fromip_table[identifier]
+        finally:
+            self._coord_lock.release()
+
+        return
+
     def establish_presence(self):
         watched_devices = self.watch_devices
 
@@ -219,7 +257,7 @@ class UpnpCoordinator(CoordinatorBase):
         self._coord_lock.acquire()
         try:
             if ifname in self._cl_iface_callback_addr_lookup:
-                callback_url = self._cl_iface_callback_addr_lookup[ifname]
+                callback_url = "<http://{}/notify>".format(self._cl_iface_callback_addr_lookup[ifname])
         finally:
             self._coord_lock.release()
 
@@ -826,22 +864,50 @@ class UpnpCoordinator(CoordinatorBase):
 
         req_body = request[headers_length:]
 
-        sid = req_headers["SID"]
-
-        device = None
-        #lookup the device that needs to handle this subscription
+        capture_to = None
         self._coord_lock.acquire()
         try:
-            if sid in self._cl_subscription_id_to_device:
-                device = self._cl_subscription_id_to_device[sid]
+            if claddr in self._cl_callback_traffic_capture_from:
+                capture_to = [cc for cc in self._cl_callback_traffic_capture_from[claddr].values()]
         finally:
             self._coord_lock.release()
 
-        if device is not None:
-            device.process_subscription_callback(claddr, sid, req_headers, req_body)
+        if capture_to is not None:
+            for cc in capture_to:
+                cc.append_capture(req_headers, req_body)
+
+        if "SID" in req_headers:
+            sid = req_headers["SID"]
+
+            device = None
+            #lookup the device that needs to handle this subscription
+            self._coord_lock.acquire()
+            try:
+                if sid in self._cl_subscription_id_to_device:
+                    device = self._cl_subscription_id_to_device[sid]
+            finally:
+                self._coord_lock.release()
+
+            if device is not None:
+                device.process_subscription_callback(claddr, sid, req_headers, req_body)
+            else:
+                errmsg = "Received subscription notification for unknown sid.  claddr={} sid={}".format(claddr, sid)
+                self._logger.error(errmsg)
         else:
-            errmsg = "Received subscription notification for unknown sid.  claddr={} sid={}".format(claddr, sid)
-            self._logger.error(errmsg)
+            dbgmsg_lines = [
+                "Received subscription notification without an SID. claddr={}".format(claddr),
+                "REQUEST HEADERS:"
+            ]
+
+            for hdr_key, hdr_val in req_headers.items():
+                dbgmsg_lines.append("    {}:{}".format(hdr_key, hdr_val))
+
+            dbgmsg_lines.append("REQUEST BODY:")
+            dbgmsg_lines.append(req_body.decode("utf-8"))
+
+            dbgmsg = os.linesep.join(dbgmsg_lines)
+
+            self._logger.debug(dbgmsg)
 
         return
 
