@@ -64,8 +64,6 @@ from akit.interop import upnp as upnp_module
 from akit.networking.constants import AKitHttpHeaders
 from akit.raisefor import raise_for_http_status
 
-from akit.xlogging.foundations import getAutomatonKitLogger
-
 # Types imported only for type checking purposes
 if TYPE_CHECKING:
     from akit.interop.upnp.upnpfactory import UpnpFactory
@@ -175,7 +173,6 @@ class UpnpRootDevice(UpnpDevice, LandscapeDeviceExtension):
         self._extra = {}
         self._cachecontrol = None
         self._ext = None
-        self._location = None
         self._server = None
         self._st = None
         self._usn = None
@@ -195,10 +192,6 @@ class UpnpRootDevice(UpnpDevice, LandscapeDeviceExtension):
         self._auto_subscribe = False
         self._mode = None
 
-        self._logger = getAutomatonKitLogger()
-
-        self._coord_ref = None
-
         self._root_device_lock = threading.RLock()
 
         self._subscriptions: Dict[StrSvcType, Tuple[StrSvcId, datetime]] = {}
@@ -402,6 +395,12 @@ class UpnpRootDevice(UpnpDevice, LandscapeDeviceExtension):
         """
         return self._usn_cls
 
+    def get_custom_subscribe_headers(self) -> Optional[Dict[str, str]]:
+        """
+            An override-able method that allows for customization of the subscribe headers.
+        """
+        return None
+
     def initialize(self, coord_ref: weakref.ReferenceType, basedevice_ref: weakref.ReferenceType, extid: str, location: str, configinfo: dict, devinfo: dict):
         """
             Initializes the landscape device extension.
@@ -767,11 +766,15 @@ class UpnpRootDevice(UpnpDevice, LandscapeDeviceExtension):
                 if callback_url is None:
                     errmsg = "No callback url found for ifname={}".format(ifname)
                     raise AKitRuntimeError(errmsg) from None
-                callback_url = "<http://%s>" % callback_url
 
                 headers = { "HOST": self._host, "User-Agent": AKitHttpHeaders.USER_AGENT, "CALLBACK": callback_url, "NT": "upnp:event"}
                 if timeout is not None:
                     headers["TIMEOUT"] = "Seconds-%s" % timeout
+
+                custom_headers = self.get_custom_subscribe_headers()
+                if custom_headers is not None:
+                    headers.update(custom_headers)
+
                 resp = requests.request(
                     "SUBSCRIBE", subscribe_url, headers=headers, auth=subscribe_auth
                 )
